/*

Ett arduino Projekt från Automatiserar.se med syfte att visa verans mode. 

Skapat av Ispep 
2015-04-24

    Reagerar på rörelse, hämtar mode ur Vera och visar nuvarande mode med en lysdiod.
    Släcker lysdioden efter 15 sekunder.
    delay på 30 sekunder mellan varje gång en rörelse får trigga enheten.
    Arduinon är även en webbserver som Vera kan skicka http kommandon till.
    Temperatur registreras i arduinon och skickas till Veran som en egen device.
    Luftfuktighet registreras i arduinon och skickas till Veran som en egen device.
    Ljusmätare registreras i arduinon och skickas till Veran som en egen device.
    Fyra adresserbara lysdioder samt program för att få rinnande ljus.

*/
#include <SPI.h>
#include <Ethernet.h>
#include <dht11.h>    // för att stödja DHT11 givaren.
//////////////////////////////////////////////////////////////////
// ALL settings for Vera MODE:
///////////////////////////////////////////////////////
    char   myVeraServer[] = "10.20.0.100"; // ange ip eftersom det verkar vara strul med DNS för tillfället...
    int    myVeraPort     = 3480;
    String myVeraURL = "/data_request?id=variableget&Variable=Mode HTTP/1.1";
    int  RowCounter;  // används för att hitta vilken rad man är på i http svaret (3 ska vara rätt!)
    String currentLine = "";  // lägger ihop allt man hittar på raden ( kanske kan tas bort senare!! )
    
    // svaret som man får från vera översatt till ascii 
    char home = 49; 
    char away = 50;
    char night= 51;
    char vacation = 52;

///////////////////////////////////////////////////////
// END ALL settings for Vera MODE:
///////////////////////////////////////////////////////    

//////////////////////////////////////////////////////////////////
// HTTP UPDATE VERA Devices 
///////////////////////////////////////////////////////    

int veraArduinoLjusID     = 69;  // Device ID på ljussensor i Vera (Arduino1 Ljus)
int veraArduinoTempID     = 68;  // ID på Temperaturgivare i Vera  (Arduino1 Temperature)
int veraArduinoFuktID     = 72;  // ID på Fuktighetsgivaren i Vera (Arduino1 Fuktighet)

///////////////////////////////////////////////////////
// END UPDATE HTTP VERA Devices 
///////////////////////////////////////////////////////    


//////////////////////////////////////////////////////////////////
// Analog LDR
///////////////////////////////////////////////////////    

const int analogInPin = A0;
int AnalogLastState = 0;
int sensorValue = 0;


///////////////////////////////////////////////////////
// END Analog LDR
///////////////////////////////////////////////////////    


//////////////////////////////////////////////////////////////////
// Fuktighet och Temperatur sensor
///////////////////////////////////////////////////////    


#define DHT11PIN 7   // definierar upp port 7 för dht11 givaren.

dht11 DHT11;
float tempgivaren;       // Innehåller data för temperaturen
float humiditygivaren;   // Innehåller data för fuktighet

///////////////////////////////////////////////////////
// END Fuktighet och Temperatur sensor
///////////////////////////////////////////////////////  

//////////////////////////////////////////////////////////////////
// LED hantering 
///////////////////////////////////////////////////////    


long ledLightTimeout = 15000; // ange hur många ms som dioderna ska vara igång efter att nått mottagets.
long lastLedTime     = 0;     // anger när den senast dioden triggade
int  ledShiftDelay   = 50;    // anger hur lång delay det ska va när den visar status uppdatering.

///////////////////////////////////////////////////////
// END LED hantering
///////////////////////////////////////////////////////    


///////////////////////////////////////////////////////
//  Ethernet settings
///////////////////////////////////////////////////////    

byte mac[] = { 0xDE, 0xAD, 0xBB, 0xAA, 0xFE, 0xED };
IPAddress ip(172, 20, 0 , 110);
byte gateway[] = {172, 20, 0, 1 }; // internet access via router
byte subnet[] = {255, 255, 0, 0 }; //subnet mask
///////////////////////////////////////////////////////
//  END Ethernet settings
///////////////////////////////////////////////////////    

// http://forum.arduino.cc/index.php?topic=134966.0

///////////////////////////////////////////////////////
//  Webserver 
///////////////////////////////////////////////////////    
EthernetServer server(80); // Server delen
boolean reading = false;   // används för att kontrollera om korrekt data mottogs.
 

EthernetClient client; // klient delen

///////////////////////////////////////////////////////
//  END Webserver 
///////////////////////////////////////////////////////    


///////////////////////////////////////////////////////
//  Button  settings
///////////////////////////////////////////////////////    

int buttonState;             // the current reading from the input pin
int lastButtonState = LOW;   // the previous reading from the input pin
long lastDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 100;    // the debounce time; increase if the output flickers (sätter den till 30, så att man inte kan trigga nått inom 30 sekunder)

const int buttonPin      =  2; // the number of the pushbutton pin

///////////////////////////////////////////////////////
//  END Button  settings
///////////////////////////////////////////////////////    

///////////////////////////////////////////////////////
//  Led Settings 
///////////////////////////////////////////////////////    
const int ledPinVacation =  3; // Orange
const int ledPinNight    =  4; // Blå
const int ledPinAway     =  5; // Red
const int ledPinHome     =  6; // Grön


///////////////////////////////////////////////////////
//  END Led Settings 
///////////////////////////////////////////////////////    


void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);


  // Webserver del 
  Ethernet.begin(mac,ip,gateway,gateway,subnet); 
  server.begin();
  // slut webserver

// Configure PIN OUTPUT AND INPUT
  pinMode(buttonPin, INPUT);
  pinMode(ledPinVacation, OUTPUT);
  pinMode(ledPinNight, OUTPUT);
  pinMode(ledPinAway, OUTPUT);
  pinMode(ledPinHome, OUTPUT);

/*
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP");
    // no point in carrying on, so do nothing forevermore:
    // try to congifure using IP address instead of DHCP:
    Ethernet.begin(mac, ip);
  }
*/  


  allLedsMode(HIGH); 
  delay(1000);
 
  Serial.println("Nu ska Ethernet vara klar...");
  Serial.println(Ethernet.localIP());
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// LOOPEN ////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop() {
  // put your main code here, to run repeatedly:
    
    
 
 
   // listen for incoming clients
    
  checkButtonstate();        
  veraWebServer(); // aggera webserver.   
  
  
  closeLedlight();  // släcker dioderna efter 60 sekunder.
  
  //readNuvarandeLjus(); // läser hur ljust det är för tillfället.
  
  //httpLjusRegister(veraArduinoLjusID, 44); // skickar in detta till veran.
  
} // END LOOP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////// END LOOPEN//////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////
// ----------------- Button CONTROL ---------------------------- //
///////////////////////////////////////////////////////////////


///// Button Code ///////
void checkButtonstate(){
  int reading = digitalRead(buttonPin);
    
    //Serial.print("Knappen har nu status: ");
    //Serial.println(reading);   
    
    if (reading != lastButtonState) {
       // reset the debouncing timer
       lastDebounceTime = millis();
     }
     
     if ((millis() - lastDebounceTime) > debounceDelay) {
     // whatever the reading is at, it's been there for longer
     // than the debounce delay, so take it as the actual current state:
  
       // if the button state has changed:
       if (reading != buttonState) {
         buttonState = reading;
         Serial.println("Knappen har bytt status");
         // only toggle the LED if the new button state is HIGH
         if (buttonState == HIGH) {
             Serial.println("Knapp nertryck, hoppar till openVeraConnection");
               openVeraConnection();
               readNuvarandeLjus();
               updateTempAndHumid(); // uppdaterar variabler med aktuell temp och fuktghet
               httpLjusRegister(veraArduinoLjusID,sensorValue);
               delay(10);
               httpFuktRegister(veraArduinoFuktID, humiditygivaren);
               delay(10);
               httpTempRegister(veraArduinoTempID, tempgivaren);
         }
       }
     }
  lastButtonState = reading;
}
///// END BUTTON CODE ////// 
////////////////////////////////////////////////////////////////
// ----------------- END Button CONTROL---------------------- //
///////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////
// ----------------- LDR Ljusmätaren ----------------------   //
///////////////////////////////////////////////////////////////

// läsar ut hur många % ljust det är för tillfället.
void readNuvarandeLjus(){
       
      sensorValue = ((analogRead(analogInPin)) /1024.00)*100;
      Serial.print("Ljuset är nu: ");
      Serial.println(sensorValue);
      //return sensorValue;          // lämnar tillbaka resultatet av sensorns värde. 
         
}
////////////////////////////////////////////////////////////////
// ----------------- END  LDR Ljusmätaren -------------------//
///////////////////////////////////////////////////////////////





///////////////////////////////////////////////////////////////////////////////////////////
// ----------------- Temp och Humidity --------------------   //
///////////////////////////////////////////////////////////////

// uppdaterar temperatur och fuktighet från dht11
void updateTempAndHumid(){
  
  
   int chk = DHT11.read(DHT11PIN); 
   
   Serial.print("Read sensor: ");
   switch (chk)
  {
    case 0: Serial.println("OK"); break;
    case -1: Serial.println("Checksum error"); break;
    case -2: Serial.println("Time out error"); break;
    default: Serial.println("Unknown error"); break;
  }
  
  Serial.print("Humidity (%): ");
  Serial.println((float)DHT11.humidity, 2);

  humiditygivaren = (float)DHT11.humidity, 2;

  Serial.print("Temperature (oC): ");
  Serial.println((float)DHT11.temperature, 2);
  
  tempgivaren = (float)DHT11.temperature, 2;
  
  
}

////////////////////////////////////////////////////////////////
// ----------------- END Temp och Humidity-----------------   //
///////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////////////////
// ----------------- LED CONTROL ---------------------------- //
///////////////////////////////////////////////////////////////

void allLedsMode(bool NewMode){
  
  digitalWrite(ledPinVacation,NewMode);
  digitalWrite(ledPinNight,NewMode);
  digitalWrite(ledPinAway,NewMode);
  digitalWrite(ledPinHome,NewMode);
  updateLedOn();  
}



// används för att visa att vera har utfört nått.
void showLedStatus(){
  
  digitalWrite(ledPinVacation,HIGH);
  delay(ledShiftDelay);
  digitalWrite(ledPinNight,HIGH);
  delay(ledShiftDelay);
  digitalWrite(ledPinAway,HIGH);
  delay(ledShiftDelay);
  digitalWrite(ledPinHome,HIGH); 
  
  digitalWrite(ledPinVacation,LOW);
  delay(ledShiftDelay);
  digitalWrite(ledPinNight,LOW);
  delay(ledShiftDelay);
  digitalWrite(ledPinAway,LOW);
  delay(ledShiftDelay);
  digitalWrite(ledPinHome,LOW); 
  
  digitalWrite(ledPinVacation,HIGH);
  delay(ledShiftDelay);
  digitalWrite(ledPinNight,HIGH);
  delay(ledShiftDelay);
  digitalWrite(ledPinAway,HIGH);
  delay(ledShiftDelay);
  digitalWrite(ledPinHome,HIGH); 
  
  openVeraConnection(); // uppdatera så att rätt mode åter sätts efter att man visat status.
  updateLedOn();        // uppdaterar så att dessa är igång i 60 sekunder
  }

////////////////////////////////////////////////////////////////////////
// --     Spara ström, stäng ledarna  ---- //
/////////////////////////////////////////////

//long ledLightTimeout = 60000; // ange hur många ms som dioderna ska vara igång efter att nått mottagets.
//long lastLedTime     = 0;     // anger när den senast dioden triggade

// DVS om det gått 60 sekunder så ska det släckas hela tiden... 
void closeLedlight(){

    unsigned long currentMillis = millis();

    if (currentMillis - lastLedTime >= ledLightTimeout){
      
       allLedsMode(LOW); 
       lastLedTime = currentMillis; 
    }
}

// denna ska alltid köras när dioder tänds!
void updateLedOn(){
  
    unsigned long currentMillis = millis();
    lastLedTime = currentMillis; 
}
/////////////////////////////////////////////
// -- END Spara ström, stäng ledarna  ---- //
/////////////////////////////////////////////


////////////////////////////////////////////////////////////////
// ----------------- END LED CONTROL------------------------- //
///////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////////////////
// ----------------- VERA MODE ----------------------------- //
///////////////////////////////////////////////////////////////
void openVeraConnection(){
  
    getCurrentVeraMode();
    while (client.connected()){
      
   if (client.available()) {
        //Serial.println(RowCounter);
        char inChar = client.read(); 
        
        currentLine += inChar; 
        
        if (inChar == '\n') { 
        RowCounter++; // lägger till så man hittar rätt rad...
        } 
        
        if (RowCounter == 3 ){
          
       
          if (inChar == home){
          Serial.println("Home");
          //currentVeraMode(ledPinHome,HIGH);
          allLedsMode(LOW);
          digitalWrite(ledPinHome,HIGH);
          } 
          else if (inChar == away){
          //currentVeraMode(ledPinAway,HIGH);
          Serial.println("Away"); 
          allLedsMode(LOW);
          digitalWrite(ledPinAway,HIGH);  
          }
          else if (inChar == night){
          //currentVeraMode(ledPinNight,HIGH);  
          Serial.println("Night");
          allLedsMode(LOW);
          digitalWrite(ledPinNight,HIGH);    
          }
          else if (inChar == vacation){
          Serial.println("Vacation");
          allLedsMode(LOW);
          digitalWrite(ledPinVacation,HIGH);          
          //currentVeraMode(ledPinVacation,HIGH);    
          } else {
          Serial.println("Annat Tecken");      
          }
                          
      }
   }

   // if the server's disconnected, stop the client:
   if (!client.connected()) {
     Serial.println();
     Serial.println("Got all data from Vera!");
     client.stop();
     RowCounter = 0; // nollar radräknaren
   }
   }
   
}

// Laddar hem data från Veran. 
void getCurrentVeraMode(){

  if (client.connect(myVeraServer, myVeraPort)) {
    Serial.println("Connected to Vera!");
    client.println("GET " + myVeraURL); // bygger början av http strängen
    client.print("Host: ");
    client.println(myVeraServer);
    
     client.println("GET " + myVeraURL);
     client.println("Host: Arduino");
     client.println("Connection: close");
     client.println();
    
     Serial.println("Skapat en uppkoppling");
  } else {
    
    Serial.println("Unable to connect to vera!");
    // stoppar i fall de inte gick bra!
    client.stop(); 
    
    }
}
////////////////////////////////////////////////////////////////
// ----------------- END VERA MODE--------------------------- //
///////////////////////////////////////////////////////////////




///////////////////////////////////////////////////////////////////////////////////////////
// ----------------- VERA Update Devices -------------------- //
///////////////////////////////////////////////////////////////

///// Ljus 

// Givarid = ditt id i vera  MittValue = vad den ska uppdatera
// Denna ska uppdatera enbart Temperatur !
void httpLjusRegister(int GivarID, int MittValue) {
      // if there's a successful connection:
      if (client.connect(myVeraServer, myVeraPort)) {
        Serial.println("Kopplar upp...");
        // send the HTTP PUT request:
        client.print("GET /data_request?id=variableset&DeviceNum=");
        client.print(GivarID);
        client.print("&serviceId=urn:micasaverde-com:serviceId:LightSensor1&Variable=CurrentLevel&Value=");
        client.print(MittValue);
        client.println(" HTTP/1.1");
        client.print("Host: ");
        client.println(myVeraServer);
        client.println("Connection: close");
        client.println();
        Serial.println("skickade Ljus till Vera, kopplar ner");
        client.stop();
        
    
      } 
      else {
        // if you couldn't make a connection:
        Serial.println("connection failed");
        Serial.println("disconnecting.");
        client.stop();
      }
    }


///// End Ljus


///// Temperatur  

// Givarid = ditt id i vera  MittValue = vad den ska uppdatera
// Denna ska uppdatera enbart Temperatur !
void httpTempRegister(int GivarID, int MittValue) {
      // if there's a successful connection:
      if (client.connect(myVeraServer, myVeraPort)) {
        Serial.println("Kopplar upp...");
        // send the HTTP PUT request:
        client.print("GET /data_request?id=variableset&DeviceNum=");
        client.print(GivarID);
        client.print("&serviceId=urn:upnp-org:serviceId:TemperatureSensor1&Variable=CurrentTemperature&Value=");
        client.print(MittValue);
        client.println(" HTTP/1.1");
        client.print("Host: ");
        client.print(myVeraServer);
        client.println("Connection: close");
        client.println();
        Serial.println("skickade Temp data till Vera, kopplar ner");
        client.stop();
        // note the time that the connection was made:
    
      } 
      else {
        // if you couldn't make a connection:
        Serial.println("connection failed");
        Serial.println("disconnecting.");
        client.stop();
      }
    }


///// End Temperatur 


//// Luftfuktighet 

    
// Givarid = ditt id i vera = mittValue = vad den ska uppdatera.    

    void httpFuktRegister(int GivarID, int MittValue) {
      // if there's a successful connection:
      if (client.connect(myVeraServer, myVeraPort)) {
        Serial.println("Kopplar upp...");
        // send the HTTP PUT request:
        client.print("GET /data_request?id=variableset&DeviceNum=");
        client.print(GivarID);
        client.print("&serviceId=urn:micasaverde-com:serviceId:HumiditySensor1&Variable=CurrentLevel&Value=");
        client.print(MittValue);
        client.println(" HTTP/1.1");
        client.print("Host: ");
        client.print(myVeraServer);
        client.println("Connection: close");
        client.println();
        Serial.println("skickade Ljus data till Vera, kopplar ner");
        client.stop();
        // note the time that the connection was made:
    
      } 
      else {
        // if you couldn't make a connection:
        Serial.println("connection failed");
        Serial.println("disconnecting.");
        client.stop();
      }
    }

//// End luftfukighet

////////////////////////////////////////////////////////////////
// ----------------- END VERA Update Devices ---------------- //
///////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////
// ----------------- Webserver - VERA ----------------------- //
///////////////////////////////////////////////////////////////

void veraWebServer(){
  

  
  //Serial.println("Inne i veraWebServer Loop");
  // listen for incoming clients
  EthernetClient client = server.available();
  
    if (client) {
    Serial.println("new client");
    // an http request ends with a blank line
    
    boolean currentLineIsBlank = true;
    boolean sentHeader = false;
    
    
    while (client.connected()) {
      if (client.available()) {
          
        if(!sentHeader){
          // send a standard http response header
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println();
          sentHeader = true;
        }
    
    char c = client.read();

        if(reading && c == ' ') reading = false;
        if(c == '?') reading = true; //found the ?, begin reading the info
        
        if(reading){
          Serial.print(c);

           switch (c) {
            case '1':
              //add code here to trigger on 1
              //triggerPin(2, client);
              digitalWrite(ledPinHome,HIGH);
              break;
              
            case '2':
            //add code here to trigger on 2
              digitalWrite(ledPinAway,HIGH);  
              break;
              
            case '3':
              //add code here to trigger on 4
              digitalWrite(ledPinNight,HIGH);  
              break;
              
            case '4':
            //triggerPin(4, client);
            digitalWrite(ledPinVacation,HIGH); 
            break;
            
            case '5':
            showLedStatus();
            break;  
            }
       }
    
       if (c == '\n' && currentLineIsBlank)  break;

        if (c == '\n') {
          currentLineIsBlank = true;
        }else if (c != '\r') {
          currentLineIsBlank = false;
        }

      }
    }

    // give the web browser time to receive the data
    delay(1);
    // close the connection:
    client.stop();
    
    Serial.println("client disconnected");
  }
  
}

////////////////////////////////////////////////////////////////
// ----------------- END webserver - VERA ------------------- //
///////////////////////////////////////////////////////////////


